package javax.swing.plaf.synth;

import java.text.NumberFormat;
import java.text.Format;
import java.text.DateFormat;
import javax.swing.border.Border;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.JCheckBox;
import java.beans.PropertyChangeEvent;
import java.awt.Component;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;
import java.awt.Rectangle;
import java.awt.Point;
import java.awt.Graphics;
import java.awt.Dimension;
import javax.swing.LookAndFeel;
import javax.swing.plaf.ColorUIResource;
import javax.swing.JTable;
import javax.swing.plaf.UIResource;
import javax.swing.ImageIcon;
import javax.swing.Icon;
import java.util.Date;
import javax.swing.plaf.ComponentUI;
import javax.swing.JComponent;
import javax.swing.table.TableCellRenderer;
import java.awt.Color;
import java.beans.PropertyChangeListener;
import javax.swing.plaf.basic.BasicTableUI;

public class SynthTableUI extends BasicTableUI implements SynthUI, PropertyChangeListener
{
    private SynthStyle style;
    private boolean useTableColors;
    private boolean useUIBorder;
    private Color alternateColor;
    private TableCellRenderer dateRenderer;
    private TableCellRenderer numberRenderer;
    private TableCellRenderer doubleRender;
    private TableCellRenderer floatRenderer;
    private TableCellRenderer iconRenderer;
    private TableCellRenderer imageIconRenderer;
    private TableCellRenderer booleanRenderer;
    private TableCellRenderer objectRenderer;
    
    public SynthTableUI() {
    }
    
    public static ComponentUI createUI(JComponent component) {
        return new SynthTableUI();
    }
    
    @Override
    protected void installDefaults() {
        this.dateRenderer = this.installRendererIfPossible(Date.class, null);
        this.numberRenderer = this.installRendererIfPossible(Number.class, null);
        this.doubleRender = this.installRendererIfPossible(Double.class, null);
        this.floatRenderer = this.installRendererIfPossible(Float.class, null);
        this.iconRenderer = this.installRendererIfPossible(Icon.class, null);
        this.imageIconRenderer = this.installRendererIfPossible(ImageIcon.class, null);
        this.booleanRenderer = this.installRendererIfPossible(Boolean.class, new SynthTableUI.SynthBooleanTableCellRenderer());
        this.objectRenderer = this.installRendererIfPossible(Object.class, new SynthTableUI.SynthTableCellRenderer());
        this.updateStyle(this.table);
    }
    
    private TableCellRenderer installRendererIfPossible(Class clazz, TableCellRenderer renderer) {
        TableCellRenderer defaultRenderer = this.table.getDefaultRenderer(clazz);
        if ((defaultRenderer instanceof UIResource)) {
            this.table.setDefaultRenderer(clazz, renderer);
            return defaultRenderer;
        }
        return defaultRenderer;
    }
    
    private void updateStyle(JTable table) {
        SynthContext context = this.getContext(table, 1);
        SynthStyle style = this.style;
        this.style = SynthLookAndFeel.updateStyle(context, this);
        if (this.style != style) {
            context.setComponentState(513);
            Color selectionBackground = this.table.getSelectionBackground();
            if (selectionBackground == null || selectionBackground instanceof UIResource) {
                this.table.setSelectionBackground(this.style.getColor(context, ColorType.TEXT_BACKGROUND));
            }
            Color selectionForeground = this.table.getSelectionForeground();
            if (selectionForeground == null || selectionForeground instanceof UIResource) {
                this.table.setSelectionForeground(this.style.getColor(context, ColorType.TEXT_FOREGROUND));
            }
            context.setComponentState(1);
            Color gridColor = this.table.getGridColor();
            if (gridColor == null || gridColor instanceof UIResource) {
                Color color = (Color)this.style.get(context, "Table.gridColor");
                if (color == null) {
                    color = this.style.getColor(context, ColorType.FOREGROUND);
                }
                this.table.setGridColor((color == null) ? new ColorUIResource(Color.GRAY) : color);
            }
            this.useTableColors = this.style.getBoolean(context, "Table.rendererUseTableColors", true);
            this.useUIBorder = this.style.getBoolean(context, "Table.rendererUseUIBorder", true);
            Object value = this.style.get(context, "Table.rowHeight");
            if (value != null) {
                LookAndFeel.installProperty(this.table, "rowHeight", value);
            }
            if (!this.style.getBoolean(context, "Table.showGrid", true)) {
                this.table.setShowGrid(false);
            }
            Dimension intercellSpacing = this.table.getIntercellSpacing();
            if (intercellSpacing != null) {
                intercellSpacing = (Dimension)this.style.get(context, "Table.intercellSpacing");
            }
            this.alternateColor = (Color)this.style.get(context, "Table.alternateRowColor");
            if (intercellSpacing != null) {
                this.table.setIntercellSpacing(intercellSpacing);
            }
            if (style != null) {
                this.uninstallKeyboardActions();
                this.installKeyboardActions();
            }
        }
        context.dispose();
    }
    
    @Override
    protected void installListeners() {
        super.installListeners();
        this.table.addPropertyChangeListener(this);
    }
    
    @Override
    protected void uninstallDefaults() {
        this.table.setDefaultRenderer(Date.class, this.dateRenderer);
        this.table.setDefaultRenderer(Number.class, this.numberRenderer);
        this.table.setDefaultRenderer(Double.class, this.doubleRender);
        this.table.setDefaultRenderer(Float.class, this.floatRenderer);
        this.table.setDefaultRenderer(Icon.class, this.iconRenderer);
        this.table.setDefaultRenderer(ImageIcon.class, this.imageIconRenderer);
        this.table.setDefaultRenderer(Boolean.class, this.booleanRenderer);
        this.table.setDefaultRenderer(Object.class, this.objectRenderer);
        if ((this.table.getTransferHandler() instanceof UIResource)) {
            this.table.setTransferHandler(null);
        }
        SynthContext context = this.getContext(this.table, 1);
        this.style.uninstallDefaults(context);
        context.dispose();
        this.style = null;
    }
    
    @Override
    protected void uninstallListeners() {
        this.table.removePropertyChangeListener(this);
        super.uninstallListeners();
    }
    
    @Override
    public SynthContext getContext(JComponent component) {
        return this.getContext(component, SynthLookAndFeel.getComponentState(component));
    }
    
    private SynthContext getContext(JComponent component, int n) {
        return SynthContext.getContext(component, this.style, n);
    }
    
    @Override
    public void update(Graphics graphics, JComponent component) {
        SynthContext context = this.getContext(component);
        SynthLookAndFeel.update(context, graphics);
        context.getPainter().paintTableBackground(context, graphics, 0, 0, component.getWidth(), component.getHeight());
        this.paint(context, graphics);
        context.dispose();
    }
    
    @Override
    public void paintBorder(SynthContext synthContext, Graphics graphics, int n, int n2, int n3, int n4) {
        synthContext.getPainter().paintTableBorder(synthContext, graphics, n, n2, n3, n4);
    }
    
    @Override
    public void paint(Graphics graphics, JComponent component) {
        SynthContext context = this.getContext(component);
        this.paint(context, graphics);
        context.dispose();
    }
    
    protected void paint(SynthContext synthContext, Graphics graphics) {
        Rectangle clipBounds = graphics.getClipBounds();
        Rectangle bounds;
        Rectangle rectangle = bounds = this.table.getBounds();
        int n = 0;
        rectangle.y = n;
        bounds.x = n;
        if (this.table.getRowCount() <= 0 || this.table.getColumnCount() <= 0 || !rectangle.intersects(clipBounds)) {
            this.paintDropLines(synthContext, graphics);
            return;
        }
        int leftToRight = this.table.getComponentOrientation().isLeftToRight();
        Point location = clipBounds.getLocation();
        Point point = new Point(clipBounds.x + clipBounds.width - 1, clipBounds.y + clipBounds.height - 1);
        int rowAtPoint = this.table.rowAtPoint(location);
        int rowAtPoint2 = this.table.rowAtPoint(point);
        if (rowAtPoint == -1) {
            rowAtPoint = 0;
        }
        if (rowAtPoint2 == -1) {
            rowAtPoint2 = this.table.getRowCount() - 1;
        }
        int columnAtPoint = this.table.columnAtPoint(leftToRight ? location : point);
        int columnAtPoint2 = this.table.columnAtPoint(leftToRight ? point : location);
        if (columnAtPoint == -1) {
            columnAtPoint = 0;
        }
        if (columnAtPoint2 == -1) {
            columnAtPoint2 = this.table.getColumnCount() - 1;
        }
        this.paintCells(synthContext, graphics, rowAtPoint, rowAtPoint2, columnAtPoint, columnAtPoint2);
        this.paintGrid(synthContext, graphics, rowAtPoint, rowAtPoint2, columnAtPoint, columnAtPoint2);
        this.paintDropLines(synthContext, graphics);
        return;
    }
    
    private void paintDropLines(SynthContext synthContext, Graphics graphics) {
        JTable.DropLocation dropLocation = this.table.getDropLocation();
        if (dropLocation == null) {
            return;
        }
        Color color = (Color)this.style.get(synthContext, "Table.dropLineColor");
        Color color2 = (Color)this.style.get(synthContext, "Table.dropLineShortColor");
        if (color == null && color2 == null) {
            return;
        }
        Rectangle hDropLineRect = this.getHDropLineRect(dropLocation);
        if (hDropLineRect != null) {
            int x = hDropLineRect.x;
            int width = hDropLineRect.width;
            if (color != null) {
                this.extendRect(hDropLineRect, true);
                graphics.setColor(color);
                graphics.fillRect(hDropLineRect.x, hDropLineRect.y, hDropLineRect.width, hDropLineRect.height);
            }
            if (!dropLocation.isInsertColumn() && color2 != null) {
                graphics.setColor(color2);
                graphics.fillRect(x, hDropLineRect.y, width, hDropLineRect.height);
            }
        }
        Rectangle vDropLineRect = this.getVDropLineRect(dropLocation);
        if (vDropLineRect != null) {
            int y = vDropLineRect.y;
            int height = vDropLineRect.height;
            if (color != null) {
                this.extendRect(vDropLineRect, false);
                graphics.setColor(color);
                graphics.fillRect(vDropLineRect.x, vDropLineRect.y, vDropLineRect.width, vDropLineRect.height);
            }
            if (!dropLocation.isInsertRow() && color2 != null) {
                graphics.setColor(color2);
                graphics.fillRect(vDropLineRect.x, y, vDropLineRect.width, height);
            }
        }
        return;
    }
    
    private Rectangle getHDropLineRect(JTable.DropLocation dropLocation) {
        if (!dropLocation.isInsertRow()) {
            return null;
        }
        int row = dropLocation.getRow();
        int column = dropLocation.getColumn();
        if (column >= this.table.getColumnCount()) {
            --column;
        }
        Rectangle cellRect = this.table.getCellRect(row, column, true);
        if (row >= this.table.getRowCount()) {
            --row;
            Rectangle cellRect2 = this.table.getCellRect(row, column, true);
            cellRect.y = cellRect2.y + cellRect2.height;
        }
        if (cellRect.y == 0) {
            cellRect.y = -1;
        }
        else {
            cellRect.y -= 2;
        }
        cellRect.height = 3;
        return cellRect;
    }
    
    private Rectangle getVDropLineRect(JTable.DropLocation dropLocation) {
        if (!dropLocation.isInsertColumn()) {
            return null;
        }
        int leftToRight = this.table.getComponentOrientation().isLeftToRight();
        int column = dropLocation.getColumn();
        Rectangle rectangle = this.table.getCellRect(dropLocation.getRow(), column, true);
        if (column >= this.table.getColumnCount()) {
            --column;
            rectangle = this.table.getCellRect(dropLocation.getRow(), column, true);
            if (!(leftToRight == 0)) {
                rectangle.x += rectangle.width;
            }
        }
        else if (leftToRight == 0) {
            rectangle.x += rectangle.width;
        }
        if (rectangle.x == 0) {
            rectangle.x = -1;
        }
        else {
            rectangle.x -= 2;
        }
        rectangle.width = 3;
        return rectangle;
    }
    
    private Rectangle extendRect(Rectangle rectangle, boolean b) {
        if (rectangle == null) {
            return rectangle;
        }
        if (b) {
            rectangle.x = 0;
            rectangle.width = this.table.getWidth();
        }
        else {
            rectangle.y = 0;
            if (this.table.getRowCount() != 0) {
                Rectangle cellRect = this.table.getCellRect(this.table.getRowCount() - 1, 0, true);
                rectangle.height = cellRect.y + cellRect.height;
            }
            else {
                rectangle.height = this.table.getHeight();
            }
        }
        return rectangle;
    }
    
    private void paintGrid(SynthContext synthContext, Graphics graphics, int i, int row, int j, int k) {
        graphics.setColor(this.table.getGridColor());
        Rectangle union = this.table.getCellRect(i, j, true).union(this.table.getCellRect(row, k, true));
        SynthGraphicsUtils graphicsUtils = synthContext.getStyle().getGraphicsUtils(synthContext);
        if (this.table.getShowHorizontalLines()) {
            int n = union.x + union.width;
            int y = union.y;
            for (; i <= row; ++i) {
                y += this.table.getRowHeight(i);
                graphicsUtils.drawLine(synthContext, "Table.grid", graphics, union.x, y - 1, n - 1, y - 1);
            }
        }
        if (this.table.getShowVerticalLines()) {
            TableColumnModel columnModel = this.table.getColumnModel();
            int n2 = union.y + union.height;
            if (this.table.getComponentOrientation().isLeftToRight()) {
                int x2 = union.x;
                for (; j <= k; ++j) {
                    x2 += columnModel.getColumn(j).getWidth();
                    graphicsUtils.drawLine(synthContext, "Table.grid", graphics, x2 - 1, 0, x2 - 1, n2 - 1);
                }
            }
            else {
                int x = union.x;
                for (; k >= j; --k) {
                    x += columnModel.getColumn(k).getWidth();
                    graphicsUtils.drawLine(synthContext, "Table.grid", graphics, x - 1, 0, x - 1, n2 - 1);
                }
            }
        }
    }
    
    private int viewIndexForColumn(TableColumn tableColumn) {
        TableColumnModel columnModel = this.table.getColumnModel();
        int i = 0;
        while (i < columnModel.getColumnCount()) {
            if (columnModel.getColumn(i) == tableColumn) {
                return i;
            }
            ++i;
        }
        return -1;
    }
    
    private void paintCells(SynthContext synthContext, Graphics graphics, int i, int n, int j, int n2) {
        JTableHeader tableHeader = this.table.getTableHeader();
        TableColumn tableColumn = (tableHeader == null) ? null : tableHeader.getDraggedColumn();
        TableColumnModel columnModel = this.table.getColumnModel();
        int columnMargin = columnModel.getColumnMargin();
        if (this.table.getComponentOrientation().isLeftToRight()) {
            Rectangle cellRect2;
            TableColumn column3;
            int width2;
            for (; i <= n; ++i) {
                for (cellRect2 = this.table.getCellRect(i, j, false), ; j <= n2; cellRect2.x += width2, ++j) {
                    column3 = columnModel.getColumn(j);
                    width2 = column3.getWidth();
                    cellRect2.width = width2 - columnMargin;
                    if (column3 != tableColumn) {
                        this.paintCell(synthContext, graphics, cellRect2, i, j);
                    }
                }
            }
        }
        else {
            Rectangle cellRect;
            TableColumn column;
            int k;
            TableColumn column2;
            int width;
            for (; i <= n; ++i) {
                cellRect = this.table.getCellRect(i, j, false);
                column = columnModel.getColumn(j);
                if (column != tableColumn) {
                    cellRect.width = column.getWidth() - columnMargin;
                    this.paintCell(synthContext, graphics, cellRect, i, j);
                }
                for (k = j + 1; k <= n2; ++k) {
                    column2 = columnModel.getColumn(k);
                    width = column2.getWidth();
                    cellRect.width = width - columnMargin;
                    cellRect.x -= width;
                    if (column2 != tableColumn) {
                        this.paintCell(synthContext, graphics, cellRect, i, k);
                    }
                }
            }
        }
        if (tableColumn != null) {
            this.paintDraggedArea(synthContext, graphics, i, n, tableColumn, tableHeader.getDraggedDistance());
        }
        this.rendererPane.removeAll();
    }
    
    private void paintDraggedArea(SynthContext synthContext, Graphics graphics, int i, int row, TableColumn tableColumn, int n) {
        int viewIndexForColumn = this.viewIndexForColumn(tableColumn);
        Rectangle union = this.table.getCellRect(i, viewIndexForColumn, true).union(this.table.getCellRect(row, viewIndexForColumn, true));
        graphics.setColor(this.table.getParent().getBackground());
        graphics.fillRect(union.x, union.y, union.width, union.height);
        union.x += n;
        graphics.setColor(synthContext.getStyle().getColor(synthContext, ColorType.BACKGROUND));
        graphics.fillRect(union.x, union.y, union.width, union.height);
        SynthGraphicsUtils graphicsUtils = synthContext.getStyle().getGraphicsUtils(synthContext);
        if (this.table.getShowVerticalLines()) {
            graphics.setColor(this.table.getGridColor());
            int x = union.x;
            int y = union.y;
            int n2 = x + union.width - 1;
            int n3 = y + union.height - 1;
            graphicsUtils.drawLine(synthContext, "Table.grid", graphics, x - 1, y, x - 1, n3);
            graphicsUtils.drawLine(synthContext, "Table.grid", graphics, n2, y, n2, n3);
        }
        Rectangle cellRect;
        Rectangle cellRect2;
        int x2;
        int y2;
        int n4;
        int n5;
        for (; i <= row; ++i) {
            cellRect = this.table.getCellRect(i, viewIndexForColumn, false);
            cellRect.x += n;
            this.paintCell(synthContext, graphics, cellRect, i, viewIndexForColumn);
            if (this.table.getShowHorizontalLines()) {
                graphics.setColor(this.table.getGridColor());
                cellRect2 = this.table.getCellRect(i, viewIndexForColumn, true);
                cellRect2.x += n;
                x2 = cellRect2.x;
                y2 = cellRect2.y;
                n4 = x2 + cellRect2.width - 1;
                n5 = y2 + cellRect2.height - 1;
                graphicsUtils.drawLine(synthContext, "Table.grid", graphics, x2, n5, n4, n5);
            }
        }
    }
    
    private void paintCell(SynthContext synthContext, Graphics g, Rectangle bounds, int row, int column) {
        if (this.table.isEditing() && this.table.getEditingRow() == row && this.table.getEditingColumn() == column) {
            Component editorComponent = this.table.getEditorComponent();
            editorComponent.setBounds(bounds);
            editorComponent.validate();
        }
        else {
            Component prepareRenderer = this.table.prepareRenderer(this.table.getCellRenderer(row, column), row, column);
            Color background = prepareRenderer.getBackground();
            if ((background == null || background instanceof UIResource || prepareRenderer instanceof SynthTableUI.SynthBooleanTableCellRenderer) && !this.table.isCellSelected(row, column) && this.alternateColor != null && row % 2 != 0) {
                prepareRenderer.setBackground(this.alternateColor);
            }
            this.rendererPane.paintComponent(g, prepareRenderer, this.table, bounds.x, bounds.y, bounds.width, bounds.height, true);
        }
    }
    
    @Override
    public void propertyChange(PropertyChangeEvent propertyChangeEvent) {
        if (SynthLookAndFeel.shouldUpdateStyle(propertyChangeEvent)) {
            this.updateStyle((JTable)propertyChangeEvent.getSource());
        }
    }
    
    private class SynthBooleanTableCellRenderer extends JCheckBox implements TableCellRenderer
    {
        private boolean isRowSelected;
        
        public SynthBooleanTableCellRenderer() {
            this.setHorizontalAlignment(0);
            this.setName("Table.cellRenderer");
        }
        
        @Override
        public Component getTableCellRendererComponent(JTable table, Object o, boolean isRowSelected, boolean b, int n, int n2) {
            this.isRowSelected = isRowSelected;
            if (isRowSelected) {
                this.setForeground(this.unwrap(table.getSelectionForeground()));
                this.setBackground(this.unwrap(table.getSelectionBackground()));
            }
            else {
                this.setForeground(this.unwrap(table.getForeground()));
                this.setBackground(this.unwrap(table.getBackground()));
            }
            this.setSelected(o != null && (boolean)o);
            return this;
        }
        
        private Color unwrap(Color color) {
            if ((color instanceof UIResource)) {
                return new Color(color.getRGB());
            }
            return color;
        }
        
        @Override
        public boolean isOpaque() {
            return this.isRowSelected || super.isOpaque();
        }
    }
    
    private class SynthTableCellRenderer extends DefaultTableCellRenderer
    {
        private Object numberFormat;
        private Object dateFormat;
        private boolean opaque;
        
        @Override
        public void setOpaque(boolean opaque) {
            this.opaque = opaque;
        }
        
        @Override
        public boolean isOpaque() {
            return this.opaque;
        }
        
        @Override
        public String getName() {
            String name = super.getName();
            if (name == null) {
                return "Table.cellRenderer";
            }
            return name;
        }
        
        @Override
        public void setBorder(Border border) {
            if (SynthTableUI.this.useUIBorder || border instanceof SynthBorder) {
                super.setBorder(border);
            }
        }
        
        @Override
        public Component getTableCellRendererComponent(JTable table, Object value, boolean isSelected, boolean hasFocus, int row, int n) {
            if (!SynthTableUI.this.useTableColors && (isSelected || hasFocus)) {
                SynthLookAndFeel.setSelectedUI((SynthLabelUI)SynthLookAndFeel.getUIOfType(this.getUI(), SynthLabelUI.class), isSelected, hasFocus, table.isEnabled(), false);
            }
            else {
                SynthLookAndFeel.resetSelectedUI();
            }
            super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, n);
            this.setIcon(null);
            if (table != null) {
                this.configureValue(value, table.getColumnClass(n));
                return this;
            }
            return this;
        }
        
        private void configureValue(Object o, Class clazz) {
            if (clazz == Object.class || clazz == null) {
                this.setHorizontalAlignment(10);
            }
            else if (clazz == Float.class || clazz == Double.class) {
                if (this.numberFormat == null) {
                    this.numberFormat = NumberFormat.getInstance();
                }
                this.setHorizontalAlignment(11);
                this.setText((o == null) ? "" : ((NumberFormat)this.numberFormat).format(o));
            }
            else if (clazz == Number.class) {
                this.setHorizontalAlignment(11);
            }
            else if (clazz == Icon.class || clazz == ImageIcon.class) {
                this.setHorizontalAlignment(0);
                this.setIcon((o instanceof Icon) ? ((Icon)o) : null);
                this.setText("");
            }
            else if (clazz == Date.class) {
                if (this.dateFormat == null) {
                    this.dateFormat = DateFormat.getDateInstance();
                }
                this.setHorizontalAlignment(10);
                this.setText((o == null) ? "" : ((Format)this.dateFormat).format(o));
            }
            else {
                this.configureValue(o, clazz.getSuperclass());
            }
        }
        
        @Override
        public void paint(Graphics g) {
            super.paint(g);
            SynthLookAndFeel.resetSelectedUI();
        }
    }
}
